@@ -92,8 +92,6 @@ _gdbm_alloc (GDBM_FILE dbf, int num_bytes)
   
 }
 
-
-
 /* Free space of size NUM_BYTES in the file DBF at file address FILE_ADR.  Make
    it avaliable for reuse through _gdbm_alloc.  This routine changes the
    avail structure. */
@@ -343,6 +341,40 @@ push_avail_block (GDBM_FILE dbf)
 
   return 0;
 }
+
+/* AV_TABLE contains COUNT entries sorted by AV_SIZE in ascending order.
+   Avail_lookup returns index of an item whose AV_SIZE member is greater
+   than or equal to SIZE. */
+static int
+avail_lookup (int size, avail_elem *av_table, int count)
+{
+  int start = 0;
+  
+  while (count > 0)
+    {
+      int pivot = start + (count >> 1);
+      if (size == av_table[pivot].av_size)
+	return pivot;
+      if (size > av_table[pivot].av_size)
+	{
+	  start = pivot + 1;
+	  count--;
+	}
+      count >>= 1;
+    }
+  return start;
+}
+
+/* In array AV_TABLE of *AV_COUNT items, move all items from
+   position SRC to DST and update *AV_COUNT accordingly.
+*/
+static inline void
+avail_move (avail_elem *av_table, int *av_count, int src, int dst)
+{
+  memmove (av_table + dst, av_table + src,
+	   (*av_count - src) * sizeof av_table[0]);
+  *av_count += dst - src;
+}
 
 /* Get_elem returns an element in the AV_TABLE block which is
    larger than SIZE.  AV_COUNT is the number of elements in the
@@ -362,11 +394,7 @@ get_elem (int size, avail_elem av_table[], int *av_count)
   val.av_size = 0;
 
   /* Search for element.  List is sorted by size. */
-  index = 0;
-  while (index < *av_count && av_table[index].av_size < size)
-    {
-      index++;
-    }
+  index = avail_lookup (size, av_table, *av_count);
 
   /* Did we find one of the right size? */
   if (index >= *av_count)
@@ -374,93 +402,57 @@ get_elem (int size, avail_elem av_table[], int *av_count)
 
   /* Ok, save that element and move all others up one. */
   val = av_table[index];
-  *av_count -= 1;
-  while (index < *av_count)
-    {
-      av_table[index] = av_table[index+1];
-      index++;
-    }
-
+  avail_move (av_table, av_count, index + 1, index);
   return val;
 }
 
-
 /* This routine inserts a single NEW_EL into the AV_TABLE block.
    This routine does no I/O. */
 
-int
+void
 _gdbm_put_av_elem (avail_elem new_el, avail_elem av_table[], int *av_count,
      		   int can_merge)
 {
-  int index;			/* For searching through the avail block. */
-  int index1;
+  int index;
 
   /* Is it too small to deal with? */
   if (new_el.av_size <= IGNORE_SIZE)
-    return FALSE;
+    return;
 
   if (can_merge == TRUE)
     {
       /* Search for blocks to coalesce with this one. */
-      index = 0;
-
-      while (index < *av_count)
+      int i;
+      
+      for (i = 0; i < *av_count; i++)
 	{
-	  /* Can we merge with the previous block? */
-	  if ((av_table[index].av_adr
-	       + av_table[index].av_size) == new_el.av_adr)
+	  if ((av_table[i].av_adr + av_table[i].av_size) == new_el.av_adr)
 	    {
-	      /* Simply expand the entry. */
-	      av_table[index].av_size += new_el.av_size;
+	      /* Right adjacent */
+	      new_el.av_size += av_table[i].av_size;
+	      new_el.av_adr = av_table[i].av_adr;
+	      avail_move (av_table, av_count, i + 1, i);
+	      --i;
+	    }
+
+	  if ((new_el.av_adr + new_el.av_size) == av_table[i].av_adr)
+	    {
+	      /* Left adjacent */
+	      new_el.av_size += av_table[i].av_size;
+	      avail_move (av_table, av_count, i + 1, i);
+	      --i;
 	    }
-	    /* Can we merge with the next block? */
-	    else if ((new_el.av_adr
-	      	      + new_el.av_size) == av_table[index].av_adr)
-	      {
-	        /* Update this entry. */
-	        av_table[index].av_adr = new_el.av_adr;
-		av_table[index].av_size += new_el.av_size;
-	      }
-	    /* Not contiguous */
-	    else
-	      {
-		index++;
-		continue;
-	      }
-	    
-	    /* If we got here, we're done. */
-	    return TRUE;
 	}
     }
 
   /* Search for place to put element.  List is sorted by size. */
-  index = 0;
-  while (index < *av_count && av_table[index].av_size < new_el.av_size)
-    {
-      index++;
-    }
-
+  index = avail_lookup (new_el.av_size, av_table, *av_count);
   /* Move all others up one. */
-  index1 = *av_count-1;
-  while (index1 >= index)
-    {
-      av_table[index1+1] = av_table[index1];
-      index1--;
-    }
-
+  avail_move (av_table, av_count, index, index + 1);
   /* Add the new element. */
   av_table[index] = new_el;
-
-  /* Increment the number of elements. */
-  *av_count += 1;
-
-  return TRUE;
 }
 
-
-
-
-
 /* Get_block "allocates" new file space and the end of the file.  This is
    done in integral block sizes.  (This helps ensure that data smaller than
    one block size is in a single block.)  Enough blocks are allocated to
@@ -505,7 +497,6 @@ adjust_bucket_avail (GDBM_FILE dbf)
     {
       if (dbf->header->avail.count > 0)
 	{
-	  //FIXME: what if _gdbm_put_av_elem return FALSE?
 	  dbf->header->avail.count -= 1;
 	  av_el = dbf->header->avail.av_table[dbf->header->avail.count];
 	  _gdbm_put_av_elem (av_el, dbf->bucket->bucket_avail,
@@ -520,14 +511,14 @@ adjust_bucket_avail (GDBM_FILE dbf)
 	 && dbf->header->avail.count < dbf->header->avail.size)
     {
       av_el = get_elem (0, dbf->bucket->bucket_avail, &dbf->bucket->av_count);
-      if (av_el.av_size == 0
-	  || _gdbm_put_av_elem (av_el, dbf->header->avail.av_table,
-				&dbf->header->avail.count,
-				dbf->coalesce_blocks) == FALSE) 
+      if (av_el.av_size == 0)
 	{
 	  GDBM_SET_ERRNO (dbf, GDBM_BAD_AVAIL, TRUE);
 	  return -1;
 	}
+      _gdbm_put_av_elem (av_el, dbf->header->avail.av_table,
+			 &dbf->header->avail.count,
+			 dbf->coalesce_blocks);
       dbf->bucket_changed = TRUE;
     }
   return 0;
